// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/Tools/Logging.hh"
#include "Rivet/Projections/FinalState.hh"
#include "Rivet/Projections/FastJets.hh"
#include <vector>
#include <iostream>
#include <cmath>
const int nevtvar=5; //Number of event shape variables
const int njetptmn=5; // Threshold of leading pt
double leadingPtThreshold[njetptmn]={110.0, 170.0, 250.0, 320.0, 390.0};
using namespace std;

namespace {

  class EventShape_rivet {

  public:

    //constructor taking as argument vectors of  
    //four vectors as input Objects    
    //in the event to calculate the event shapes

    EventShape_rivet(std::vector<double> Px_vector, std::vector<double> Py_vector, std::vector<double> Pz_vector,std::vector<double> E_vector, double eta_central, int rapidity, int nmn);

    //Destructor
    ~EventShape_rivet(){};

    //0. central transverse thrust             
    //1. central total jet broadening
    //2. central total jet mass       
    //3. central total transverse jet mass 
    //4. central three-jet resolution threshold

    //returns the values of the event shapes 
    std::vector<double> getEventShapes();

    std::vector<double> getThrustAxis(); //returns the global thrust axis Nx, Ny, Nz=0  
    std::vector<double> getThrustAxis_C();  //returns the central thrust axis Nx, Ny, Nz=0

    //eta_c: choice of the central region
    //recommended: the two hardest jets should be within the central region
    void SetEtac(double eta_central){
      eta_c = eta_central;
    }

    //wether to take the rapidity y (rap==1)  or the pseudorapidity eta (rap==0)
    void SetMethod(int rapidity){
      rap = rapidity;
    }

  private:

    int calculate();
    std::vector<double> testval;
    std::vector<double> Object_Px;
    std::vector<double> Object_Py;
    std::vector<double> Object_Pz;
    std::vector<double> Object_P;
    std::vector<double> Object_Pt;
    std::vector<double> Object_E;
    std::vector<double> Object_Phi;
    std::vector<double> Object_Eta;
    std::vector<double> EventShapes;
    std::vector<double> ThrustAxis;
    std::vector<double> ThrustAxis_c;

    double eta_c;
    int rap;
    int y3_recom;
    unsigned int nmnjet;

    //returns the difference in phi between two vectors
    double DeltaPhi(double, double);

    double max (double, double);

    double min (double, double);

    //the lorentz scalar product
    double lorentz_sp (std::vector<double>, std::vector<double>);

    //calculates the three-jet resolutions    
    double three_jet_res(std::vector<double>,std::vector<double>,std::vector<double>,std::vector<double>, int);

    //calculates the thrust axis and the tau values
    std::vector<double> Thrust_calculate(std::vector<double>, std::vector<double>);

  };
  
  EventShape_rivet::EventShape_rivet(vector<double> Px_vector, vector<double> Py_vector, vector<double> Pz_vector, vector<double> E_vector, double eta_central, int rapidity, int nmn) : Object_Px(Px_vector),Object_Py(Py_vector), Object_Pz(Pz_vector), Object_E(E_vector), eta_c(eta_central),rap(rapidity), nmnjet(nmn){}
  
  vector<double> EventShape_rivet::getEventShapes(){
    this->calculate();
    return EventShapes;
  }

  vector<double> EventShape_rivet::getThrustAxis(){
    this->calculate();
    return ThrustAxis;
  }

  vector<double> EventShape_rivet::getThrustAxis_C(){
    this->calculate();
    return ThrustAxis_c;
  }

  int EventShape_rivet::calculate(){

    unsigned int length = (unsigned int) Object_Px.size();

    if(((unsigned int) Object_Py.size()!=length)||((unsigned int) Object_Pz.size()!=length)||((unsigned int) Object_E.size()!=length)){
      cout << "ERROR!!!! Input vectors differ in size! Change that please!" << endl;
      cout<<"py_size: "<<Object_Py.size()<<" ,pz_size: "<<Object_Pz.size()
	  <<" ,px_size: "<<Object_Px.size()<<" ,E_size: "<<Object_E.size()<<endl;
      return 0;
    }

    if (!Object_P.empty()){
      Object_P.clear();
      Object_Pt.clear();
      Object_Eta.clear();
      Object_Phi.clear();
      EventShapes.clear();
      ThrustAxis.clear();
      //    ThrustAxis_c.clear();          
    }
    for(unsigned int j = 0; j < length; j++){
      Object_P.push_back(0.);
      Object_Pt.push_back(0.);
      Object_Eta.push_back(0.);
      Object_Phi.push_back(0.);
    }

    for(int j = 0; j < nevtvar; j++){
      EventShapes.push_back(-50.);
    }

    EventShapes.push_back(double(Object_Px.size()));

    for(unsigned int j = 0; j < 3; j++){
      ThrustAxis.push_back(0.);
      ThrustAxis_c.push_back(0.);
    }

    double theta=0;

    for(unsigned int k =0; k<length; k++){
      Object_P[k] = sqrt(pow(Object_Px[k],2) + pow(Object_Py[k],2) + pow(Object_Pz[k],2));
      Object_Pt[k] = sqrt(pow(Object_Px[k],2) + pow(Object_Py[k],2));
      if(Object_P[k]>Object_E[k] + 1E-4){
	cout << "ERROR!!! Object " << k <<" has P = " << Object_P[k]
	     << " which is bigger than E = " << Object_E[k] <<" "
	     << Object_Px[k] <<" "<< Object_Py[k] <<" "
	     << Object_Pz[k] <<" of total length "<< length
	     <<endl;
	return 0;
      }

      //to prevent a division by zero         
      if(rap==0){
	if (fabs(Object_Pz[k]) > 1E-5) {
	  theta = atan(Object_Pt[k]/(Object_Pz[k]));
	} else {
	  theta = M_PI/2;
	}
	if(theta<0.) {theta = theta + M_PI;}
	Object_Eta[k] = - log(tan(0.5*theta));
      }
      if(rap==1){
	if(Object_Pz[k]==Object_E[k]){
	  cout<<"ERROR!!! Object "<<k<<" has Pz "<< Object_Pz[k] <<" which is equal to E = "<< Object_E[k] <<endl;
	  return 0;
	}
	Object_Eta[k]=0.5*log((Object_E[k]+Object_Pz[k])/(Object_E[k]-Object_Pz[k]));
      }
      if((rap!=0)&&(rap!=1)){
	cout << "ERROR!!!, The choice to use the rapidity y or the pseudorapidity eta is not set correctly! Change that please!" << endl;
	return 0;
      }
      Object_Phi[k] = atan2(Object_Py[k], Object_Px[k]);
    }

    //||||||||||||||||||||||||||||||||||||||||||||||
    //here the central event shape variables begin||  
    //||||||||||||||||||||||||||||||||||||||||||||||   

    std::vector<double> Object_Px_in;
    std::vector<double> Object_Py_in;
    std::vector<double> Object_Pz_in;
    std::vector<double> Object_Pt_in;
    std::vector<double> Object_E_in;
    std::vector<double> Object_Et_in;
    std::vector<double> Object_Eta_in;
    std::vector<double> Object_Px_out;
    std::vector<double> Object_Py_out;
    std::vector<double> Object_Pz_out;
    std::vector<double> Object_E_out;
    std::vector<double> Object_Pt_out;
    std::vector<double> Object_Eta_out;

    if (!Object_Px_in.empty()){
      Object_Px_in.clear();
      Object_Py_in.clear();
      Object_Pz_in.clear();
      Object_Pt_in.clear();
      Object_E_in.clear();
      Object_Et_in.clear();
      Object_Eta_in.clear();
      Object_Px_out.clear();
      Object_Py_out.clear();
      Object_Pz_out.clear();
      Object_Pt_out.clear();
      Object_E_out.clear();
      Object_Eta_out.clear();
    }

    unsigned int nin = 0;

    for(unsigned int j=0;j<length; j++){
      if(fabs(Object_Eta[j])<eta_c){
	Object_Px_in.push_back(Object_Px[j]);
	Object_Py_in.push_back(Object_Py[j]);
	Object_Pz_in.push_back(Object_Pz[j]);
	Object_E_in.push_back(Object_E[j]);
	Object_Pt_in.push_back(sqrt(pow(Object_Px[j],2)+pow(Object_Py[j],2)));
	Object_Et_in.push_back(sqrt((pow(Object_E[j],2)*pow(Object_Pt[j],2))/(pow(Object_Pt[j],2)+pow(Object_Pz[j],2))));
	Object_Eta_in.push_back(Object_Eta[j]);
	nin+=1;}
      else{
	Object_Px_out.push_back(Object_Px[j]);
	Object_Py_out.push_back(Object_Py[j]);
	Object_Pz_out.push_back(Object_Pz[j]);
	Object_E_out.push_back(Object_E[j]);
	Object_Pt_out.push_back(sqrt(pow(Object_Px[j],2)+pow(Object_Py[j],2)));
	Object_Eta_out.push_back(Object_Eta[j]);
      }
    }
    if(Object_Px_in.size() != nin){
      cout<<"ERROR!!! wrong dimension of in momenta"<<endl;
    }

    unsigned int nout = length - nin;

    if(nin<nmnjet){
      for(int i=0; i<nevtvar; i++){
	EventShapes[i]=-50.0;
      }
    }

    EventShapes[nevtvar] = nin;

    if (nin>=nmnjet){
      double P_sum_c = 0; //GMA 
      double Pt_sum_c = 0;
      double Eta_cw=0;
      double px_sum_in = 0;
      double py_sum_in = 0;
      for(unsigned int j=0;j<nin;j++){
	Pt_sum_c += Object_Pt_in[j];
	P_sum_c += sqrt(pow(Object_Pt_in[j],2.) + pow(Object_Pz_in[j], 2.0)); //GMA 
	Eta_cw += Object_Pt_in[j]*Object_Eta_in[j];
	px_sum_in += Object_Px_in[j];
	py_sum_in += Object_Py_in[j];
      }
      Eta_cw = Eta_cw / Pt_sum_c;

      double ExpTerm =0;
      for(unsigned int j=0;j<nout;j++){
	ExpTerm += Object_Pt_out[j] * exp(-fabs(Object_Eta_out[j]-Eta_cw));
      }
      ExpTerm = ExpTerm / Pt_sum_c;

      //the central global transverse thrust centrthr is calculated    
      double centrthr=0;

      std::vector<double> Thrust_central = Thrust_calculate(Object_Px_in, Object_Py_in);

      for(unsigned int l=0; l<3; l++){
	ThrustAxis_c[l]=Thrust_central[l];
      }
      //the variable which gets resummed is not thrust 
      //but tau = 1 - thrust - see calculation     
      centrthr=Thrust_central[3];
      EventShapes[0] = centrthr;

      double alpha_c = atan2(ThrustAxis_c[1], ThrustAxis_c[0]);
      //central jet masses    
      //define two jet masses in region U and D   
      double cenjm_up = 0;
      double cenjm_down= 0;
      double dot_product = 0;

      std::vector<double> up_sum;
      std::vector<double> down_sum;
      for(unsigned int j=0; j<4;j++){
	up_sum.push_back(0.);
	down_sum.push_back(0.);
      }
      for(unsigned int i=0;i<nin;i++){
	dot_product = Object_Px_in[i] * ThrustAxis_c[0] + Object_Py_in[i] * ThrustAxis_c[1];
	if(dot_product>=0){
	  up_sum[0]+=Object_Px_in[i];
	  up_sum[1]+=Object_Py_in[i];
	  up_sum[2]+=Object_Pz_in[i];
	  up_sum[3]+=Object_E_in[i];
	}else{
	  down_sum[0]+=Object_Px_in[i];
	  down_sum[1]+=Object_Py_in[i];
	  down_sum[2]+=Object_Pz_in[i];
	  down_sum[3]+=Object_E_in[i];
	}
      }
      cenjm_up = lorentz_sp(up_sum, up_sum) / pow(P_sum_c, 2.); //GMA pow(Pt_sum_c,2);    
      cenjm_down = lorentz_sp(down_sum, down_sum) / pow(P_sum_c, 2.); //GMA pow(Pt_sum_c,2);  

      //central total jet mass centotjm    
      double centotjm=0;
      centotjm = cenjm_up+cenjm_down;

      EventShapes[2]=centotjm;

      double centrjm_up=0;
      double centrjm_down=0;
      std::vector<double> upsum;
      std::vector<double> downsum;
      for(unsigned int j=0; j<3;j++){
	upsum.push_back(0.);
	downsum.push_back(0.);
      }
      for(unsigned int i=0;i<nin;i++){
	dot_product = Object_Px_in[i]*ThrustAxis_c[0]+Object_Py_in[i]*ThrustAxis_c[1];
	if(dot_product>=0){
	  upsum[0] += Object_Px_in[i];
	  upsum[1] += Object_Py_in[i];
	  upsum[2] += Object_Et_in[i];
	}else{
	  downsum[0] += Object_Px_in[i];
	  downsum[1] += Object_Py_in[i];
	  downsum[2] += Object_Et_in[i];
	}
      }
      centrjm_up= lorentz_sp(upsum, upsum) / pow(Pt_sum_c, 2);
      centrjm_down= lorentz_sp(downsum, downsum) / pow(Pt_sum_c, 2);
      double centottrjm = centrjm_up + centrjm_down;

      EventShapes[3] = centottrjm;

      //central three-jet resolution threshold 
      double ceny3=0;
      if(nin<3){
	ceny3 =-1.0;
      } else{
	ceny3 = three_jet_res(Object_Px_in, Object_Py_in, Object_Pz_in, Object_E_in, rap);
      }

      EventShapes[4] = ceny3;

      //the central jet broadenings in the up and down    
      //region       
      double cenbroad_up=0;
      double cenbroad_down=0;

      double eta_up=0;
      unsigned int num_up=0;
      double eta_down =0;
      unsigned int num_down =0;
      double phi_temp =0;
      double phi_up_aver =0;
      double phi_down_aver =0;
      double Pt_sum_up =0;
      double Pt_sum_down =0;
      double dot_product_b =0;
      vector<double> phi_up;
      vector<double> phi_down;
      double py_rot =0;
      double px_rot =0;

      for(unsigned int j=0; j<4;j++){
	up_sum.push_back(0.);
	down_sum.push_back(0.);
      }

      for(unsigned int i=0;i<nin;i++){
	dot_product_b =sqrt(Object_Px_in[i]*ThrustAxis_c[0]+
			    Object_Py_in[i]*ThrustAxis_c[1]);
	if(dot_product_b>=0){
	  Pt_sum_up += Object_Pt_in[i];
	  //rotate the coordinate system so that       
	  //the central thrust axis is e_x         
	  px_rot = cos(alpha_c)*Object_Px_in[i]+sin(alpha_c)*Object_Py_in[i];
	  py_rot = - sin(alpha_c)*Object_Px_in[i]+cos(alpha_c)*Object_Py_in[i];
	  //calculate the eta and phi in the rotated system   
	  eta_up+=Object_Pt_in[i]*Object_Eta_in[i];
	  phi_temp =atan2(py_rot,px_rot);

	  if(phi_temp>M_PI/2){
	    phi_temp = phi_temp - M_PI/2;
	  }
	  if (phi_temp<-M_PI/2){
	    phi_temp = phi_temp + M_PI/2;
	  }
	  phi_up.push_back(phi_temp);
	  phi_up_aver+=Object_Pt_in[i]*phi_temp;
	  num_up+=1;
	}else{
	  eta_down+=Object_Pt_in[i]*Object_Eta_in[i];
	  Pt_sum_down+=Object_Pt_in[i];
	  px_rot = cos(alpha_c)*Object_Px_in[i]+sin(alpha_c)*Object_Py_in[i];
	  py_rot = - sin(alpha_c)*Object_Px_in[i]+cos(alpha_c)*Object_Py_in[i];
	  phi_temp =atan2(py_rot,px_rot);
	  if(phi_temp>M_PI/2){
	    //if phi is bigger than pi/2 in the new system calculate 
	    //the difference to the thrust axis   
	    phi_temp = M_PI -phi_temp;
	  }
	  if (phi_temp<-M_PI/2){
	    //if phi is smaller than   
	    phi_temp = -M_PI-phi_temp;
	  }
	  phi_down.push_back(phi_temp);
	  //calculate the pt-weighted phi    
	  phi_down_aver+=Object_Pt_in[i]*phi_temp;
	  num_down+=1;
	}
      }
      if (num_up!=0){
	eta_up=eta_up/Pt_sum_up;
	phi_up_aver=phi_up_aver/Pt_sum_up;
      }
      if(num_down!=0){
	eta_down = eta_down/Pt_sum_down;
	phi_down_aver=phi_down_aver/Pt_sum_down;
      }

      unsigned int index_up=0;
      unsigned int index_down=0;


      for(unsigned int i=0;i<nin;i++){
      dot_product_b = Object_Px_in[i]*ThrustAxis_c[0]+
        Object_Py_in[i]*ThrustAxis_c[1];
      if(dot_product_b>=0){
        //calculate the broadenings of the regions with the rotated system   
        //and the pt-weighted average of phi in the rotated system    
        cenbroad_up += Object_Pt_in[i]*sqrt(pow(Object_Eta_in[i]-eta_up, 2) +
                                            pow(DeltaPhi(phi_up[index_up],
                                                         phi_up_aver), 2));
        index_up+=1;
      }else{
        cenbroad_down += Object_Pt_in[i]*sqrt(pow(Object_Eta_in[i]-eta_down, 2)+
                                              pow(DeltaPhi(phi_down[index_down],
                                                           phi_down_aver), 2));
        index_down+=1;
      }
      }

      if (index_up == 0 || index_down ==0) EventShapes[nevtvar] *=-1.;

      cenbroad_up=cenbroad_up/(2*Pt_sum_c);
      cenbroad_down=cenbroad_down/(2*Pt_sum_c);

      //central total jet broadening     
      double centotbroad=0;
      centotbroad = cenbroad_up+cenbroad_down;

      EventShapes[1]=centotbroad;

      for (int ij=0; ij<5; ij++) {
	if (EventShapes[ij] < 1.e-20) EventShapes[ij] = 1.e-20;
	EventShapes[ij] = log(EventShapes[ij]);
      }
    }

    return 1;
  }



  //||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||   
  //function which calculates the three-jet resolution thresholds||| 
  //||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  

  double EventShape_rivet::three_jet_res(std::vector<double> In_Object_Px, std::vector<double> In_Object_Py, std::vector<double> In_Object_Pz, std::vector<double> In_Object_E, int rap){

    unsigned int y3_length = (unsigned int)In_Object_Px.size();

    if(((unsigned int) In_Object_Py.size()!=y3_length)||((unsigned int) In_Object_Pz.size()!=y3_length)||(In_Object_E.size()!=y3_length)){
      cout << "ERROR!!!! Input vectors differ in size! Change that please!" << endl;
      cout<<"py_size: "<<In_Object_Py.size()<<" ,pz_size: "<<In_Object_Pz.size()
	  <<" ,px_size: "<<In_Object_Px.size()<<" , E_size: "<<In_Object_E.size() <<endl;
      return 0.0;
    }

    std::vector<double> In_Object_P;
    std::vector<double> In_Object_Pt;
    std::vector<double> In_Object_Eta;
    std::vector<double> In_Object_Phi;

    if (!In_Object_P.empty()){
      In_Object_P.clear();
      In_Object_Pt.clear();
      In_Object_Eta.clear();
      In_Object_Phi.clear();
    }


    for(unsigned int j = 0; j < y3_length; j++){
      In_Object_P.push_back(0.);
      In_Object_Pt.push_back(0.);
      In_Object_Eta.push_back(0.);
      In_Object_Phi.push_back(0.);
    }
    double theta_y3_1st=0;
    for(unsigned int k =0; k<y3_length; k++){
      In_Object_P[k] = sqrt(pow(In_Object_Px[k],2) + pow(In_Object_Py[k],2) + pow(In_Object_Pz[k],2));
      In_Object_Pt[k] = sqrt(pow(In_Object_Px[k],2) + pow(In_Object_Py[k],2));

      //calculates the pseudorapidity    
      //to prevent a division by zero    
      if(rap==0){
	if (fabs(In_Object_Pz[k]) > 1E-5) {
	  theta_y3_1st = atan(In_Object_Pt[k]/(In_Object_Pz[k]));
	}else{
	  theta_y3_1st = M_PI/2;
	}
	if(theta_y3_1st<0.){
	  theta_y3_1st = theta_y3_1st + M_PI;
	}
	In_Object_Eta[k] = - log(tan(0.5*theta_y3_1st));
      }
      //calculates the real rapidity  
      if(rap==1){
	In_Object_Eta[k]=0.5*log((In_Object_E[k]+In_Object_Pz[k])/(In_Object_E[k]-In_Object_Pz[k]));
      }
      In_Object_Phi[k] = atan2(In_Object_Py[k], In_Object_Px[k]);
    }

    //the three-jet resolution   
    //threshold y3 
    double y3 =0;

    //vector which will be filled with the   
    //minimum of the distances    
    double max_dmin_temp=0;

    double max_dmin =0;

    //distance input object k, beam    
    double distance_jB=0;
    double distance_jB_min=0;
    //distance of input object k to l   
    double distance_jk=0;
    double distance_jk_min =0;
    //as we search the minimum of the distances  
    //give them values which are for sure higher  
    //than those we evaluate first in the for-loups 

    unsigned int index_jB=0;

    unsigned int index_j_jk = 0;
    unsigned int index_k_jk =0;

    //to decide later if the minmum is a jB or jk   
    int decide_jB=-1;

    std::vector<double> input_Pt;
    std::vector<double> input_Px;
    std::vector<double> input_Py;
    std::vector<double> input_Pz;
    std::vector<double> input_P;
    std::vector<double> input_E;
    std::vector<double> input_Phi;
    std::vector<double> input_Eta;

    if (!input_Pt.empty()){
      input_Pt.clear();
      input_Px.clear();
      input_Px.clear();
      input_Pz.clear();
      input_P.clear();
      input_E.clear();
      input_Phi.clear();
      input_Eta.clear();
    }

    for(unsigned int j=0; j<y3_length;j++){
      input_Pt.push_back(In_Object_Pt[j]);
      input_Px.push_back(In_Object_Px[j]);
      input_Py.push_back(In_Object_Py[j]);
      input_Pz.push_back(In_Object_Pz[j]);
      input_P.push_back(In_Object_P[j]);
      input_E.push_back(In_Object_E[j]);
      input_Phi.push_back(In_Object_Phi[j]);
      input_Eta.push_back(In_Object_Eta[j]);
    }
    if(y3_length<3){
      //cout<<"WARNING: less than three input momenta!! The directly global three-jet resolution threshold will be set to -1.0 per default!"<<endl;

      return -1.; }

    else{
      unsigned int rest = y3_length;
      for(unsigned int i = 0; i<y3_length; i++){
	//make the minima at the initialization step 
	//of each louping bigger than the first values  
	distance_jB_min = 0.36*pow(input_Pt[0],2) + 10;
	//DELTA PHIs wanted not the pure difference      
	distance_jk_min=min(pow(input_Pt[1], 2), pow(input_Pt[0], 2)) *
	  (pow(input_Eta[1]-input_Eta[0], 2) +
	   pow(DeltaPhi(input_Phi[1], input_Phi[0]), 2)) + 10;
	//do the procedure only until we have only 2 objects left anymore   
	if(rest>2) {
	  for(unsigned int j=0; j<rest;j++){
	    //calculate the distance between object j and the beam 
	    distance_jB = 0.36*pow(input_Pt[j], 2);
	    if(distance_jB < distance_jB_min){
	      distance_jB_min = distance_jB;
	      index_jB = j;
	    }
	    if(j > 0){
	      for(unsigned int k=0; k<j;k++){
		//calculate the distance in delta eta and delta phi 
		//between object i and object j  
		distance_jk = min(pow(input_Pt[j], 2),pow(input_Pt[k], 2))*
		  (pow(input_Eta[j]-input_Eta[k], 2)+
		   pow(DeltaPhi(input_Phi[j],input_Phi[k]), 2));
		if(distance_jk<distance_jk_min){
		  distance_jk_min = distance_jk;
		  index_j_jk = j;
		  index_k_jk =k;
		}
	      }
	      //cout<<"distance jk min "<<distance_jk_min<<endl;    
	    }
	  }
	  //decide if the minimum is from a jB  
	  //or jk combination                                       
	  if(distance_jk_min<distance_jB_min){
	    max_dmin_temp = max(distance_jk_min,max_dmin_temp);
	    decide_jB = 0;
	  }else{
	    max_dmin_temp = max(distance_jB_min,max_dmin_temp);
	    decide_jB=1;}
	  //if we have only three jets left calculate   
	  //the maxima of the dmin's   
	  //if the minimum is a jB eliminate the input object     
	  if(decide_jB==1){
	    //if index_jB is the last one nothing is to do    
	    if(index_jB!=rest-1){
	      for(unsigned int i=index_jB; i<rest-1;i++){
		input_Pt[i]=input_Pt[i+1];
		input_Phi[i]=input_Phi[i+1];
		input_Eta[i]=input_Eta[i+1];
		input_Px[i]=input_Px[i+1];
		input_Py[i]=input_Py[i+1];
		input_Pz[i]=input_Pz[i+1];
		input_E[i]=input_E[i+1];
	      }
	    }
	  }
	  //if the minimum is a jk combine both input objects     
	  if(decide_jB==0) {
	    input_Px[index_k_jk] = input_Px[index_k_jk]+input_Px[index_j_jk];
	    input_Py[index_k_jk] = input_Py[index_k_jk]+input_Py[index_j_jk];
	    input_Pz[index_k_jk] = input_Pz[index_k_jk]+input_Pz[index_j_jk];
	    input_E[index_k_jk] = input_E[index_k_jk]+input_E[index_j_jk];
	    input_P[index_k_jk] = sqrt(pow(input_Px[index_k_jk], 2)+
				       pow(input_Py[index_k_jk], 2)+
				       pow(input_Pz[index_k_jk], 2));
	    //calculate the pt, eta and phi of the new combined momenta k_jk  
	    input_Pt[index_k_jk] = sqrt(pow(input_Px[index_k_jk], 2)+
					pow(input_Py[index_k_jk], 2));
	    //in the case of pseudorapidity  
	    if(rap==0){
	      double theta_new =0;
	      if (fabs(input_Pz[index_k_jk]) > 1E-5){
		theta_new = atan(input_Pt[index_k_jk]/(input_Pz[index_k_jk]));
	      }else{
		theta_new = M_PI/2;
	      }
	      if(theta_new<0.){
		theta_new = theta_new + M_PI;
	      }
	      input_Eta[index_k_jk] = - log(tan(0.5*theta_new));
	    }
	    //in the real rapidity y is wanted 
	    if(rap==1){
	      input_Eta[index_k_jk] = 0.5 * log((input_E[index_k_jk]+
						 input_Pz[index_k_jk]) /
						(input_E[index_k_jk] -
						 input_Pz[index_k_jk]));
	    }
	    input_Phi[index_k_jk] = atan2(input_Py[index_k_jk], input_Px[index_k_jk]);
	    if(index_j_jk!=rest-1){
	      for(unsigned int i=index_j_jk; i<rest-1;i++){
		input_Pt[i]=input_Pt[i+1];
		input_Phi[i]=input_Phi[i+1];
		input_Eta[i]=input_Eta[i+1];
		input_Px[i]=input_Px[i+1];
		input_Py[i]=input_Py[i+1];
		input_Pz[i]=input_Pz[i+1];
		input_E[i]=input_E[i+1];
	      }
	    }
	  }
	}
	if(rest==3){
	  max_dmin=max_dmin_temp;
	}
	rest=rest-1;
      }
    }

    double Et2=0;
    Et2= input_Pt[0]+input_Pt[1];

    y3=max_dmin/pow(Et2,2);

    return y3;
  }



  //|||||||||||||||||||||||||||||||||||||||||    
  //function which calculates the thrusts||||  
  //|||||||||||||||||||||||||||||||||||||||||     

  std::vector<double> EventShape_rivet::Thrust_calculate (std::vector<double> Input_Px, std::vector<double> Input_Py) {

    double thrustmax_calc =0;
    double temp_calc =0;
    unsigned int length_thrust_calc =0;
    std::vector<double> ThrustValues;
    std::vector<double> Thrust_Axis_calc;
    std::vector<double> p_thrust_max_calc;
    std::vector<double> p_dec_1_calc;
    std::vector<double> p_dec_2_calc;
    std::vector<double> p_pt_beam_calc;

    if (!ThrustValues.empty()){
      ThrustValues.clear();
      Thrust_Axis_calc.clear();
      p_thrust_max_calc.clear();
      p_dec_1_calc.clear();
      p_dec_2_calc.clear();
      p_pt_beam_calc.clear();
    }

    for(unsigned int j = 0; j < 3; j++){
      p_pt_beam_calc.push_back(0.);
      p_dec_1_calc.push_back(0.);
      p_dec_2_calc.push_back(0.);
      p_thrust_max_calc.push_back(0.);
      Thrust_Axis_calc.push_back(0.);
    }

    for(unsigned int j =0;j<4;j++){
      ThrustValues.push_back(0.);
    }

    //  cout <<"lenght "<< Input_Px.size()<<endl; 
    length_thrust_calc = Input_Px.size();
    if(Input_Py.size()!=length_thrust_calc){
      cout<<"ERROR in thrust calculation!!! Size of Input vectors differs. Change that please!"<<endl;
      return ThrustValues;
    }

    float Pt_sum_calc =0;

    for(unsigned int k=0;k<length_thrust_calc;k++){
      Pt_sum_calc+=sqrt(pow(Input_Px[k],2)+pow(Input_Py[k],2));
      for(unsigned int j = 0; j < 3; j++){
	p_thrust_max_calc[j]=0;
      }
      //get a vector perpendicular to the beam axis and   
      //perpendicular to the momentum of particle k 
      //per default beam axis b = (0,0,1)
      p_pt_beam_calc[0] = Input_Py[k]*1;
      p_pt_beam_calc[1] = - Input_Px[k]*1;
      p_pt_beam_calc[2] = 0.; // GMA p_pt_beam_calc[3] = 0.;  
      for(unsigned int i=0;i<length_thrust_calc;i++){
	if(i!=k){
	  if((Input_Px[i]*p_pt_beam_calc[0]+Input_Py[i]*p_pt_beam_calc[1])>=0){
	    p_thrust_max_calc[0]= p_thrust_max_calc[0] + Input_Px[i];
	    p_thrust_max_calc[1]= p_thrust_max_calc[1] + Input_Py[i];
	  }else{
	    p_thrust_max_calc[0]= p_thrust_max_calc[0] - Input_Px[i];
	    p_thrust_max_calc[1]= p_thrust_max_calc[1] - Input_Py[i];
	  }
	}
      }
      p_dec_1_calc[0]=p_thrust_max_calc[0] + Input_Px[k];
      p_dec_1_calc[1]=p_thrust_max_calc[1] + Input_Py[k];
      p_dec_1_calc[2]=0;
      p_dec_2_calc[0]=p_thrust_max_calc[0] - Input_Px[k];
      p_dec_2_calc[1]=p_thrust_max_calc[1] - Input_Py[k];
      p_dec_2_calc[2]=0;
      temp_calc = pow(p_dec_1_calc[0], 2) + pow(p_dec_1_calc[1], 2);

      if(temp_calc>thrustmax_calc){
	thrustmax_calc =temp_calc;
	for(unsigned int i=0;i<3;i++){
	  Thrust_Axis_calc[i]=p_dec_1_calc[i]/sqrt(thrustmax_calc);
	}
      }
      temp_calc = pow(p_dec_2_calc[0], 2)+pow(p_dec_2_calc[1], 2);
      if(temp_calc>thrustmax_calc){
	thrustmax_calc =temp_calc;
	for(unsigned int i=0;i<3;i++){
	  Thrust_Axis_calc[i]=p_dec_2_calc[i]/sqrt(thrustmax_calc);
	}
      }
    }
    for(unsigned int j=0;j<3;j++){
      ThrustValues[j]=Thrust_Axis_calc[j];
    }
    double thrust_calc=0;
    thrust_calc = sqrt(thrustmax_calc)/Pt_sum_calc;

    //the variable which gets resummed is not the thrust 
    //but tau=1-thrust 
    ThrustValues[3]=1.-thrust_calc;

    if (ThrustValues[3] < 1.e-20) ThrustValues[3] = 1.e-20;

    return ThrustValues;
  }

  double EventShape_rivet::DeltaPhi(double phi1, double phi2)
  {
    double delta_phi = fabs(phi2 - phi1);
    if (delta_phi > M_PI){
      delta_phi = 2*M_PI-delta_phi;
    }
    return delta_phi;
  }

  double EventShape_rivet::max (double a, double b)
  {
    if(a>=b){
      return a;
    }else{
      return b;
    }
  }

  double EventShape_rivet::min (double a, double b)
  {
    if(a<=b){
      return a;
    }else{
      return b;
    }
  }

  //returns the scalar product between two 4 momenta          
  double EventShape_rivet::lorentz_sp(std::vector<double> a, std::vector<double> b){
    unsigned int dim = (unsigned int) a.size();
    if(a.size()!=b.size()){
      cout<<"ERROR!!! Dimension of input vectors are different! Change that please!"<<endl;
      return 0;
    }else{
      double l_dot_product=a[dim-1]*b[dim-1];
      for(unsigned int i=0; i<dim-1;i++){
	l_dot_product-=a[i]*b[i];
      }
      return l_dot_product;
    }
  }
}

namespace Rivet {


  class CMS_2014_I1305624 : public Analysis {
  public:

    /// Constructor 
    CMS_2014_I1305624()
      : Analysis("CMS_2014_I1305624")
    {    }

  public:

    /// @name Analysis methods 

    /// Book histograms and initialise projections before the run 
    void init() {
      nevt = 0;

      /// @todo Initialise and register projections here    
      const FastJets jets(FinalState(-2.6, 2.6, 0.0*GeV),
                          FastJets::ANTIKT, 0.5);
      addProjection(jets, "Jets");

      /// @todo Book histograms here, e.g.:    
      for (int ij=0; ij<njetptmn; ij++) {
        _h_thrustc[ij] = bookHisto1D(1, 1, ij+1);
        _h_broadt[ij] = bookHisto1D(1, 2, ij+1);
        _h_tot3dmass[ij] = bookHisto1D(1, 3, ij+1);
        _h_tottrnsmass[ij] = bookHisto1D(1, 4, ij+1);
        _h_y23c[ij] = bookHisto1D(1, 5, ij+1);
        // 
        _alow1[ij] = _h_thrustc[ij]->xMin();
        _alow2[ij] = _h_broadt[ij]->xMin();
        _alow3[ij] = _h_tot3dmass[ij]->xMin();
        _alow4[ij] = _h_tottrnsmass[ij]->xMin();
        _alow5[ij] = _h_y23c[ij]->xMin();

        _ahgh1[ij] = _h_thrustc[ij]->xMax();
        _ahgh2[ij] = _h_broadt[ij]->xMax();
        _ahgh3[ij] = _h_tot3dmass[ij]->xMax();
        _ahgh4[ij] = _h_tottrnsmass[ij]->xMax();
        _ahgh5[ij] = _h_y23c[ij]->xMax();

      }
    }

    /// Perform the per-event analysis   
    void analyze(const Event& event) {
      nevt++;
      const double weight = event.weight();

      /// @todo Do the event by event analysis here   

      const Jets& jets = applyProjection<FastJets>(event, "Jets").jetsByPt(30.0*GeV);

      if (jets.size()<2) vetoEvent;
      double leadingpt = jets[0].momentum().pT() / GeV;
      if (jets.size() < 2 ||
          fabs(jets[0].momentum().eta()) >= 2.4 ||
          fabs(jets[1].momentum().eta()) >= 2.4 ||
          leadingpt < 110.0) {

        vetoEvent;
      }

      std::vector<double> jtpx;
      std::vector<double> jtpy;
      std::vector<double> jtpz;
      std::vector<double> jten;
      foreach (const Jet& ij, jets) {
	if (fabs(ij.momentum().eta()) < 2.4) {
          jtpx.push_back(ij.momentum().px());
          jtpy.push_back(ij.momentum().py());
          jtpz.push_back(ij.momentum().pz());
          jten.push_back(ij.momentum().E());
        }
      }

      EventShape_rivet  eventshape(jtpx, jtpy, jtpz, jten, 2.4, 0, 2);
      eventvar = eventshape.getEventShapes();

      if (eventvar[nevtvar]<0) vetoEvent; // Jets are not only one hemesphere    

      for (int ij=njetptmn-1; ij>=0; ij--) {

        if (leadingpt > leadingPtThreshold[ij]) {
          if (eventvar[0]>=_alow1[ij] &&  eventvar[0]<=_ahgh1[ij]) {
            _h_thrustc[ij]->fill(eventvar[0], weight);
          }
          if (eventvar[2]>=_alow3[ij] &&  eventvar[2]<=_ahgh3[ij]) {
            _h_tot3dmass[ij]->fill(eventvar[2], weight);
          }
          if (eventvar[3]>=_alow4[ij] &&  eventvar[3]<=_ahgh4[ij]) {
            _h_tottrnsmass[ij]->fill(eventvar[3], weight);
          }
          if (eventvar[nevtvar]>=3) {
            if (eventvar[1]>=_alow2[ij] &&  eventvar[1]<=_ahgh2[ij]) {
              _h_broadt[ij]->fill(eventvar[1], weight);
            }
            if (eventvar[4]>=_alow5[ij] &&  eventvar[4]<=_ahgh5[ij]) {
              _h_y23c[ij]->fill(eventvar[4], weight);
            }
          }
          break;
        }
      }
    }
    /// Normalise histograms etc., after the run    
    void finalize() {
      /// @todo Normalise, scale and otherwise manipulate histograms here     

      for (int ij=0; ij<njetptmn; ij++) {
        normalize(_h_thrustc[ij]);
        normalize(_h_broadt[ij]);
        normalize(_h_tot3dmass[ij]);
        normalize(_h_tottrnsmass[ij]);
        normalize(_h_y23c[ij]);
      }
    }

  private:

    // Data members like post-cuts event weight counters go here     

  private:
    /// @name Histograms 

    Histo1DPtr _h_thrustc[njetptmn];
    Histo1DPtr _h_broadt[njetptmn];
    Histo1DPtr _h_tot3dmass[njetptmn];
    Histo1DPtr _h_tottrnsmass[njetptmn];
    Histo1DPtr _h_y23c[njetptmn];

    vector<double>eventvar;
    int nevt;


    double _alow1[njetptmn], _alow2[njetptmn], _alow3[njetptmn], _alow4[njetptmn], _alow5[njetptmn];
    double _ahgh1[njetptmn], _ahgh2[njetptmn], _ahgh3[njetptmn], _ahgh4[njetptmn], _ahgh5[njetptmn];
  };

  // The hook for the plugin system
  DECLARE_RIVET_PLUGIN(CMS_2014_I1305624);
  
}
